﻿using System.Collections.Generic;
using System.Linq;
using System.ComponentModel;
using System.Text;
using IdeaBlade.EntityModel;

// This sample demostrates the use of a stored procedure query asynchronously

namespace SilverlightConsole {
  public class CodeSample {
    public CodeSample() {
    }

    public void Start() {
      AsyncStoredProcQuery();
    }

    private async void AsyncStoredProcQuery() {
      SLConsole.WriteLine("Note: The server may take a few seconds to initialize.\n");

      // Execute a stored procedure query asynchronously

      // Get an Employee so we can pass his ID as a parameter in the stored procedure query
      var employee = await _entityManager.Employees
        .OrderBy(e => e.LastName)
        .AsScalarAsync()
        .First();

      SLConsole.WriteLine("The employee retrieved is " + employee.FirstName + " " + employee.LastName +
        "; ID# " + employee.EmployeeID);
      SLConsole.WriteLine("Retrieving " + employee.FirstName + " " + employee.LastName + 
        "'s orders via Stored Procedure Query.");

      // We can pass the stored proc arguments as parameters to the query.  
      // Here we'll query for orders for the employee retrieved in the previous query in the year 1996.
      StoredProcQuery storedProcQuery = _entityManager.EmployeeOrdersForYearQuery(employee.EmployeeID, 1996);

      // The StoredProcQuery is an IEntityQuery, not an IEntityQuery<T>, so we need to cast the untyped results.
      var orders = (await storedProcQuery.ExecuteAsync()).Cast<Order>();

      SLConsole.WriteLine("Retrieved " + orders.Count() + " orders:");
      foreach (var order in orders) {
        SLConsole.WriteLine("Order " + order.OrderID);
      }

      // Let's also show navigation properties here.  We can't do an Include with the StoredProcQuery, but we can lazily and 
      // asynchronously load related data. In Silverlight, asynchronous navigation is used to load related data into cache.  
      // Note that we'll issue two async queries here (via async navigation). Since the queries will complete independent 
      // of one another, the message list showing the results will not be in any particular order.
      
      var firstOrder = orders.First();
      SLConsole.WriteLine("\nRetrieving Customer and Line items for first order (" + firstOrder.OrderID + ")...");

      // Let's get the Customer for the first Order
      firstOrder.Customer.EntityAspect.PendingEntityResolved += (o, e) => {
        SLConsole.WriteLine("Retrieved customer '" + ((Customer)e.ResolvedEntity).CompanyName + "'");
      };

      // We'll also retrieve the OrderDetails for the first order. We will listen on the PendingEntityListResolved event for such.
      firstOrder.OrderDetails.PendingEntityListResolved += (o, e) => {
        SLConsole.WriteLine("Retrieved " + e.ResolvedEntities.Count + " line items:");
        foreach (var orderDetail in firstOrder.OrderDetails) {
          SLConsole.WriteLine("Line item " + orderDetail.OrderID + "," + orderDetail.ProductID);
        }
      };
    }

    private NorthwindIBEntities _entityManager = new NorthwindIBEntities();
  }
}
